@@ -1,25 +1,30 @@
-# MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
-
-# MySQL Connector/Python is licensed under the terms of the GPLv2
-# <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-# MySQL Connectors. There are special exceptions to the terms and
-# conditions of the GPLv2 as it is applied to this software, see the
-# FOSS License Exception
-# <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+# Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License, version 2.0, as
+# published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an
+# additional permission to link the program and your derivative works
+# with the separately licensed software that they have included with
+# MySQL.
+#
+# Without limiting anything contained in the foregoing, this file,
+# which is part of MySQL Connector/Python, is also subject to the
+# Universal FOSS Exception, version 1.0, a copy of which can be found at
+# http://oss.oracle.com/licenses/universal-foss-exception.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# See the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
 """Implementing communication with MySQL servers.
 """
@@ -59,7 +64,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._converter_class = MySQLConverter
 
         self._client_flags = ClientFlag.get_default()
-        self._charset_id = 33
+        self._charset_id = 45
         self._sql_mode = None
         self._time_zone = None
         self._autocommit = False
@@ -90,12 +95,15 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._auth_plugin = None
         self._pool_config_version = None
 
-        if len(kwargs) > 0:
+        self._columns_desc = []
+
+        if kwargs:
             try:
                 self.connect(**kwargs)
             except:
                 # Tidy-up underlying socket on failure
                 self.close()
+                self._socket = None
                 raise
 
     def _do_handshake(self):
@@ -108,8 +116,9 @@ class MySQLConnection(MySQLConnectionAbstract):
         try:
             handshake = self._protocol.parse_handshake(packet)
         except Exception as err:
-            raise errors.InterfaceError(
-                'Failed parsing handshake; {0}'.format(err))
+            # pylint: disable=E1101
+            raise errors.get_mysql_exception(msg=err.msg, errno=err.errno,
+                                             sqlstate=err.sqlstate)
 
         self._server_version = self._check_server_version(
             handshake['server_version_original'])
@@ -128,7 +137,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._handshake = handshake
 
     def _do_auth(self, username=None, password=None, database=None,
-                 client_flags=0, charset=33, ssl_options=None):
+                 client_flags=0, charset=45, ssl_options=None):
         """Authenticate with the MySQL server
 
         Authentication happens in two parts. We first send a response to the
@@ -147,7 +156,10 @@ class MySQLConnection(MySQLConnectionAbstract):
                                        ssl_options.get('cert'),
                                        ssl_options.get('key'),
                                        ssl_options.get('verify_cert') or False,
-                                       ssl_options.get('cipher'))
+                                       ssl_options.get('verify_identity') or
+                                       False,
+                                       ssl_options.get('cipher'),
+                                       ssl_options.get('version', None))
             self._ssl_active = True
 
         packet = self._protocol.make_auth(
@@ -170,6 +182,8 @@ class MySQLConnection(MySQLConnectionAbstract):
         Raises NotSupportedError when we get the old, insecure password
         reply back. Raises any error coming from MySQL.
         """
+        auth = None
+        new_auth_plugin = self._auth_plugin or self._handshake["auth_plugin"]
         packet = self._socket.recv()
         if packet[4] == 254 and len(packet) == 5:
             raise errors.NotSupportedError(
@@ -185,12 +199,22 @@ class MySQLConnection(MySQLConnectionAbstract):
             response = auth.auth_response()
             self._socket.send(response)
             packet = self._socket.recv()
-            if packet[4] != 1:
-                return self._handle_ok(packet)
-            else:
-                auth_data = self._protocol.parse_auth_more_data(packet)
+
+        if packet[4] == 1:
+            auth_data = self._protocol.parse_auth_more_data(packet)
+            auth = get_auth_plugin(new_auth_plugin)(
+                auth_data, password=password, ssl_enabled=self._ssl_active)
+            if new_auth_plugin == "caching_sha2_password":
+                response = auth.auth_response()
+                if response:
+                    self._socket.send(response)
+                packet = self._socket.recv()
+
+        if packet[4] == 0:
+            return self._handle_ok(packet)
         elif packet[4] == 255:
             raise errors.get_exception(packet)
+        return None
 
     def _get_connection(self, prtcls=None):
         """Get connection based on configuration
@@ -200,7 +224,6 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Returns subclass of MySQLBaseSocket.
         """
-        # pylint: disable=R0204
         conn = None
         if self.unix_socket and os.name != 'nt':
             conn = MySQLUnixSocket(unix_socket=self.unix_socket)
@@ -208,7 +231,7 @@ class MySQLConnection(MySQLConnectionAbstract):
             conn = MySQLTCPSocket(host=self.server_host,
                                   port=self.server_port,
                                   force_ipv6=self._force_ipv6)
-        # pylint: enable=R0204
+
         conn.set_connection_timeout(self._connection_timeout)
         return conn
 
@@ -221,15 +244,20 @@ class MySQLConnection(MySQLConnectionAbstract):
         """
         self._protocol = MySQLProtocol()
         self._socket = self._get_connection()
-        self._socket.open_connection()
-        self._do_handshake()
-        self._do_auth(self._user, self._password,
-                      self._database, self._client_flags, self._charset_id,
-                      self._ssl)
-        self.set_converter_class(self._converter_class)
-        if self._client_flags & ClientFlag.COMPRESS:
-            self._socket.recv = self._socket.recv_compressed
-            self._socket.send = self._socket.send_compressed
+        try:
+            self._socket.open_connection()
+            self._do_handshake()
+            self._do_auth(self._user, self._password,
+                          self._database, self._client_flags, self._charset_id,
+                          self._ssl)
+            self.set_converter_class(self._converter_class)
+            if self._client_flags & ClientFlag.COMPRESS:
+                self._socket.recv = self._socket.recv_compressed
+                self._socket.send = self._socket.send_compressed
+        except:
+            # close socket
+            self.close()
+            raise
 
     def shutdown(self):
         """Shut down connection to MySQL Server.
@@ -241,7 +269,6 @@ class MySQLConnection(MySQLConnectionAbstract):
             self._socket.shutdown()
         except (AttributeError, errors.Error):
             pass  # Getting an exception would mean we are disconnected.
-        self._socket = None
 
     def close(self):
         """Disconnect from the MySQL server"""
@@ -250,10 +277,9 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         try:
             self.cmd_quit()
-            self._socket.close_connection()
         except (AttributeError, errors.Error):
             pass  # Getting an exception would mean we are disconnected.
-        self._socket = None
+        self._socket.close_connection()
 
     disconnect = close
 
@@ -420,16 +446,16 @@ class MySQLConnection(MySQLConnectionAbstract):
         if not column_count or not isinstance(column_count, int):
             raise errors.InterfaceError('Illegal result set.')
 
-        columns = [None,] * column_count
+        self._columns_desc = [None,] * column_count
         for i in range(0, column_count):
-            columns[i] = self._protocol.parse_column(
+            self._columns_desc[i] = self._protocol.parse_column(
                 self._socket.recv(), self.python_charset)
 
         eof = self._handle_eof(self._socket.recv())
         self.unread_result = True
-        return {'columns': columns, 'eof': eof}
+        return {'columns': self._columns_desc, 'eof': eof}
 
-    def get_row(self, binary=False, columns=None):
+    def get_row(self, binary=False, columns=None, raw=None):
         """Get the next rows returned by the MySQL server
 
         This method gets one row from the result set after sending, for
@@ -439,12 +465,13 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Returns a tuple.
         """
-        (rows, eof) = self.get_rows(count=1, binary=binary, columns=columns)
-        if len(rows):
+        (rows, eof) = self.get_rows(count=1, binary=binary, columns=columns,
+                                    raw=raw)
+        if rows:
             return (rows[0], eof)
         return (None, eof)
 
-    def get_rows(self, count=None, binary=False, columns=None):
+    def get_rows(self, count=None, binary=False, columns=None, raw=None):
         """Get all rows returned by the MySQL server
 
         This method gets all rows returned by the MySQL server after sending,
@@ -453,13 +480,19 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Returns a tuple()
         """
+        if raw is None:
+            raw = self._raw
+
         if not self.unread_result:
             raise errors.InternalError("No result set available.")
 
         try:
             if binary:
+                charset = self.charset
+                if charset == 'utf8mb4':
+                    charset = 'utf8'
                 rows = self._protocol.read_binary_result(
-                    self._socket, columns, count)
+                    self._socket, columns, count, charset)
             else:
                 rows = self._protocol.read_text_result(self._socket,
                                                        self._server_version,
@@ -468,13 +501,19 @@ class MySQLConnection(MySQLConnectionAbstract):
             self.unread_result = False
             raise err
 
-        if rows[-1] is not None:
-            row = rows[-1]  # OK or EOF
-            self._handle_server_status(row['status_flag'] if 'status_flag' in
-                                       row else row['server_status'])
+        rows, eof_p = rows
+
+        if not (binary or raw) and self._columns_desc is not None and rows \
+           and hasattr(self, 'converter'):
+            row_to_python = self.converter.row_to_python
+            rows = [row_to_python(row, self._columns_desc) for row in rows]
+
+        if eof_p is not None:
+            self._handle_server_status(eof_p['status_flag'] if 'status_flag' in
+                                       eof_p else eof_p['server_status'])
             self.unread_result = False
 
-        return rows
+        return rows, eof_p
 
     def consume_results(self):
         """Consume results
@@ -651,7 +690,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         return self._handle_ok(self._send_cmd(ServerCmd.PING))
 
     def cmd_change_user(self, username='', password='', database='',
-                        charset=33):
+                        charset=45):
         """Change the current logged in user
 
         This method allows to change the current logged in user information.
